/*
 * Released under MIT license terms, read LICENSE file
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "psensor.h"
#include "graph.h"

GtkWidget* w_graph_box;

struct psensor** sensors;

struct psensor** get_all_sensors() 
{
	int colors[6][3] = { 
		{1,0,0}, 
		{0,1,0}, 
		{0,0,1}, 
		{1,0,1},
		{0,1,1},
		{1,1,0}
	};

	struct psensor** psensors = NULL;
	int count = 0;
	const sensors_chip_name *chip;
	int chip_nr = 0;

	while ((chip = sensors_get_detected_chips(NULL, &chip_nr))) {
		const sensors_feature *feature;
		int i = 0;
		while ((feature = sensors_get_features(chip, &i))) {
			struct psensor* psensor;

			if (feature->type ==  SENSORS_FEATURE_TEMP
			    && (psensor = create_sensor(chip, feature))) {
				psensor->color[0] = colors[count%6][0];
				psensor->color[1] = colors[count%6][1];
				psensor->color[2] = colors[count%6][2];

				struct psensor** tmp_psensors;
				tmp_psensors 
					= malloc((count+1+1)*sizeof(struct psensor*));
				
				memcpy(tmp_psensors, 
				       psensors, 
				       count*sizeof(struct psensor*));

				tmp_psensors[count] = psensor;
				tmp_psensors[count+1] = NULL;
				
				free(psensors);
				psensors = tmp_psensors;
				count++;
			}	
		}
	}

	return psensors;
}


void update_information() 
{
	while(1) {
		struct psensor** sensor_cur = sensors;
		while(*sensor_cur) {
			gdk_threads_enter();
			update_psensor(*sensor_cur);
			gdk_threads_leave();
			sensor_cur++;
		}
		sleep(1);
	}
}


gboolean graph_refresh_thread() 
{
	update_graph(sensors,w_graph_box);

	return TRUE;
}



gboolean on_expose_event (GtkWidget *widget, 
			  GdkEventExpose *event, 
			  gpointer user_data) 
{
	update_graph(sensors,w_graph_box);

	return FALSE;
}


int main(int argc, char** argv) 
{
	printf("Psensor - Temperature Monitor - v0.1.0 by wpitchoune@gmail.com\n");
	printf("Released under MIT license terms, read LICENSE file\n");
	printf("http://wpitchoune.net:9969/psensor\n");

	int err = sensors_init(NULL);
	if (err) {
		fprintf(stderr, "ERROR: %s\n", sensors_strerror(err));
		return 1;
	}

	sensors = get_all_sensors();

	g_thread_init(NULL);
	gdk_threads_init();
	gdk_threads_enter();

	gtk_init (&argc, &argv);

	GtkWidget* w_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width (GTK_CONTAINER (w_window), 2);
	gtk_window_set_default_size(GTK_WINDOW(w_window),800,200);
	gtk_window_set_title(GTK_WINDOW(w_window), "Psensor - Temperature Monitor");

	//////////// main box in the window
	GtkWidget* mbox = gtk_hbox_new(FALSE,0);
	gtk_container_add (GTK_CONTAINER (w_window), mbox);


	GtkDrawingArea* drawing_area = GTK_DRAWING_AREA(gtk_drawing_area_new ());
	w_graph_box = GTK_WIDGET(drawing_area);
	gtk_box_pack_start(GTK_BOX(mbox), GTK_WIDGET(drawing_area), TRUE, TRUE, 0);

	g_signal_connect(G_OBJECT(drawing_area), 
			 "expose-event",
			 G_CALLBACK(on_expose_event), NULL);

	///////////// sensor list
	GtkWidget* sbox = gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(mbox), GTK_WIDGET(sbox), 
			   FALSE, FALSE, 0);       
	
	struct psensor** sensor_cur = sensors;
	while(*sensor_cur) {
		GtkWidget* label = gtk_label_new((*sensor_cur)->name);

		GdkColor color = {
			0,
			(*sensor_cur)->color[0]*65535,
			(*sensor_cur)->color[1]*65535,
			(*sensor_cur)->color[2]*65535
		};
		
		gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &color);

		gtk_box_pack_start(GTK_BOX(sbox), GTK_WIDGET(label), 
				   FALSE, FALSE, 0);       
		sensor_cur++;
	}

	gtk_widget_show_all (w_window);

	GError* error;
	GThread* thread = g_thread_create((GThreadFunc)update_information,
			NULL, TRUE, &error);

	if(!thread) 
		g_error_free(error);
	
	g_timeout_add (1000, graph_refresh_thread, clock);

	gtk_main ();
	gdk_threads_leave();

	sensors_cleanup();

	return 0;
}
